use clap::{Arg, ArgAction, Command};
use mox::vm::InterpretResult;
use mox::vm::VM;
use std::fs;
use std::io;
use std::io::Write;
use std::process;
use std::process::exit;

fn main() {
    let app = Command::new("mox")
        .version("1.0")
        .author("publicmatt")
        .about("mox interpreter!!")
        .arg(
            Arg::new("file")
                .short('f')
                .long("file")
                .action(ArgAction::Set)
                .value_name("FILE")
                .help("file to run"),
        )
        .arg(
            Arg::new("command")
                .short('c')
                .long("command")
                .action(ArgAction::Set)
                .value_name("COMMAND")
                .help("command to run"),
        );

    let matches = app.get_matches();

    if let Some(command) = matches.get_one::<String>("command") {
        run_content(command);
    } else if let Some(file) = matches.get_one::<String>("file") {
        run_file(file);
    } else {
        repl();
    }
}

fn repl() {
    let mut input = String::new();
    let mut vm: VM = VM::new();
    loop {
        input.clear();
        print!("> ");
        io::stdout().flush().unwrap();

        match io::stdin().read_line(&mut input) {
            Ok(bytes) => {
                if bytes == 0 || input.trim() == "exit" {
                    println!("Bye!");
                    break;
                }
                vm.interpret(&input);
            }
            Err(_error) => {
                continue;
            }
        }
    }
}

fn run_content(source: &str) {
    let mut vm: VM = VM::new();
    match vm.interpret(source) {
        InterpretResult::InterpretOk => exit(0),
        InterpretResult::InterpretCompileError => exit(65),
        InterpretResult::InterpretRuntimeError => exit(70),
    }
}

fn run_file(path: &str) {
    let mut vm: VM = VM::new();
    let content = fs::read_to_string(path).unwrap_or_else(|err| {
        eprintln!("Error reading '{}': {}", path, err);
        process::exit(74);
    });

    match vm.interpret(&content) {
        InterpretResult::InterpretOk => exit(0),
        InterpretResult::InterpretCompileError => exit(65),
        InterpretResult::InterpretRuntimeError => exit(70),
    }
}
